/*	Copyright (C) 2003-2013 Free Electron Organization
	Any use of this software requires a license.  If a valid license
	was not distributed with this file, visit freeelectron.org. */

/** @file */

#include "vegetation.pmh"

namespace fe
{

DrawPlant::DrawPlant(void)
{
	setIdentity(m_up[0]);
	rotate(m_up[0],0.5*M_PI,e_yAxis);

	setIdentity(m_up[1]);
	rotate(m_up[1],-0.5*M_PI,e_xAxis);

	setIdentity(m_up[2]);
}

void DrawPlant::handle(Record& record)
{
	m_forestRV.bind(record);
	sp<DrawI> spDrawI=m_forestRV.drawI();
	if(spDrawI.isNull())
	{
		return;
	}

	if(!m_plantRV.scope().isValid())
	{
		sp<Scope> spScope=record.layout()->scope();
		m_plantRV.bind(spScope);
	}

	const I32 upAxis=m_forestRV.upAxis();
	const F32 deltaT=m_forestRV.deltaT();
#ifdef FE_VEG_VIEWER
	sp<ControllerI> spControllerI=m_forestRV.controllerI();
#endif

	sp<RecordGroup> spRG=m_forestRV.plants();
	FEASSERT(spRG.isValid());

	for(RecordGroup::iterator it=spRG->begin();it!=spRG->end();it++)
	{
		sp<RecordArray> spRA= *it;
		if(!m_plantRV.plantModel.check(spRA))
			continue;

		m_plantRV.bind(spRA);
		while(m_plantRV.step())
		{
			sp<SketchI> spSketchI=m_plantRV.plantSketch();
			if(!spSketchI.isValid())
			{
				spSketchI=registry()->create("*.PlantSketch");
				FEASSERT(m_plantRV.plantModel().isValid());
				m_plantRV.plantSketch()=spSketchI;

				spSketchI->bind(m_plantRV.plantModel());
#ifdef FE_VEG_VIEWER
				spSketchI->bind(spControllerI);
#endif
			}
			FEASSERT(spSketchI.isValid());
			spSketchI->setTransform(m_up[upAxis]);
			spSketchI->setPaused(deltaT==0.0f);
			spSketchI->draw(spDrawI);
		}
	}
}

} // namespace


